@@ -6,15 +6,15 @@ import bz2
 from contextlib import contextmanager
 import io
 import json
-import logging
 import os
 import queue
 from queue import Empty
 import re
-import sys
 import time
 import threading
 
+from jsoncache.log import get_logger
+
 import boto3
 from botocore.client import Config
 
@@ -33,16 +33,7 @@ CLOUD_TYPES = {
     "gcs": "gcs",
 }
 
-
-# Setup a logger for this module
-logger = logging.getLogger()
-logger.setLevel(logging.DEBUG)
-
-handler = logging.StreamHandler(sys.stdout)
-handler.setLevel(logging.INFO)
-formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
-handler.setFormatter(formatter)
-logger.addHandler(handler)
+CHUNK_SIZE = 100 * 1000  # 100k byte reads at a time
 
 
 @contextmanager
@@ -55,7 +46,8 @@ def context_timer(msg):
         end_load = time.time()
         load_time = end_load - start_load
         msg += f"  Load Time: {load_time} sec"
-        logger.debug(msg)
+        # Setup a logger for this module
+        get_logger("jsoncache").info(msg)
 
 
 def decode_payload(payload, path):
@@ -85,18 +77,33 @@ def s3_json_loader(bucket, path, region_name="us-west-2"):
 
     On any error loading from the path, return None
     """
+    logger = get_logger("jsoncache")
     try:
-        msg = f"Loaded s3://{bucket}/{path}."
+        s3_fullpath = f"s3://{bucket}/{path}"
+        msg = f"Loaded {s3_fullpath}"
         with context_timer(msg):
             config = Config(
                 connect_timeout=10,
                 region_name=region_name,
                 retries={"max_attempts": 3},
             )
-            s3 = boto3.client("s3", config=config)
+            s3 = boto3.resource("s3", config=config)
+
+            bucket = s3.Bucket(bucket)
+            s3_obj = bucket.Object(path).get()
+            s3_stream = s3_obj["Body"]
 
             with io.BytesIO() as file_out:
-                s3.download_fileobj(bucket, path, file_out)
+                bytes_read = 0
+                while True:
+                    chunk = s3_stream.read(CHUNK_SIZE)
+                    bytes_read += len(chunk)
+                    if chunk:
+                        logger.info(f"Read {bytes_read} bytes from {s3_fullpath}")
+                        file_out.write(chunk)
+                    else:
+                        break
+
                 file_out.seek(0)
                 payload = file_out.read()
                 payload = decode_payload(payload, path)
@@ -129,7 +136,7 @@ def gcs_json_loader(bucket, path):
 
                 return payload
     except Exception:
-        logger.exception(f"Error loading from gcs://{bucket}/{path}")
+        get_logger("jsoncache").exception(f"Error loading from gcs://{bucket}/{path}")
 
     return None
 
@@ -153,7 +160,7 @@ class Timer:
         future = self._start + (self._ttl_incr * self._ttl)
 
         expired = now > future
-        logger.debug(
+        get_logger("jsoncache").debug(
             f"Incr: {self._ttl_incr} Comparing time: {now} > {future} == {expired}"
         )
 
@@ -223,8 +230,8 @@ class ThreadedObjectCache:
         self._cached_result = None
 
         assert self._cloud_type in CLOUD_TYPES
-        assert BUCKET_RE.match(self._bucket)
-        assert PATH_RE.match(self._path)
+        assert BUCKET_RE.match(self._bucket), "Invalid bucket name"
+        assert PATH_RE.match(self._path), "Invalid path for storage"
 
         # Note that the refresh thread is daemonized so that the
         # thread dies when the process dies
@@ -274,6 +281,7 @@ class ThreadedObjectCache:
         return self._cached_result
 
     def _dequeue_result(self):
+        logger = get_logger("jsoncache")
         while True:
             if self._stop:
                 break
@@ -283,7 +291,10 @@ class ThreadedObjectCache:
                 time.sleep(1)
                 continue
 
-            assert result is not None
+            if result is None:
+                logger.error("Ignoring None dequeued to clobber cached value.")
+                continue
+
             # We've dequeued a result - clobber the current instance
             logger.debug("Writing new model to cache")
             self._cached_result = result
@@ -294,6 +305,8 @@ class ThreadedObjectCache:
         """
         This method is called by the background `result_thread` to
         load data from the cloud
+
+        On error, this method will return None
         """
         if self._cloud_type == CLOUD_TYPES["s3"]:
             return s3_json_loader(self._bucket, self._path, self._transformer)
@@ -305,6 +318,7 @@ class ThreadedObjectCache:
         Loop forever checking the expiry time and pushing a message into
         the refresh_queue if a model should be reloaded
         """
+        logger = get_logger("jsoncache")
         t = Timer(clock, ttl)
         while True:
             if self._stop:
@@ -316,6 +330,7 @@ class ThreadedObjectCache:
         logger.info(f"refresh_thread stopped. thread:{threading.get_ident()}")
 
     def result_thread(self, model_loader, refresh_queue, result_queue, transformer):
+        logger = get_logger("jsoncache")
         while True:
             if self._stop:
                 break
@@ -325,7 +340,17 @@ class ThreadedObjectCache:
                 time.sleep(1)
                 continue
 
-            result = model_loader()
+            result = None
+            while True:
+                result = model_loader()
+                # It's possible we get some kind of network failure,
+                # just try again immediately
+                if result is not None:
+                    break
+                logger.warn(
+                    f"Retrying download for {self._cloud_type}://{self._bucket}/{self._path}"
+                )
+
             logger.debug("Model is loaded")
             if transformer is not None:
                 logger.debug("Transform being applied")
